@@ -98,6 +98,9 @@
 #include "gst_private.h"
 #include "gstdevicemonitor.h"
 
+GST_DEBUG_CATEGORY_STATIC (devicemonitor_debug);
+#define GST_CAT_DEFAULT devicemonitor_debug
+
 struct _GstDeviceMonitorPrivate
 {
   gboolean started;
@@ -187,6 +190,8 @@ gst_device_monitor_class_init (GstDeviceMonitorClass * klass)
   object_class->set_property = gst_device_monitor_set_property;
   object_class->dispose = gst_device_monitor_dispose;
 
+  GST_DEBUG_CATEGORY_INIT (devicemonitor_debug, "devicemonitor", 0,
+      "debugging info for the device monitor");
   g_object_class_install_property (object_class, PROP_SHOW_ALL,
       g_param_spec_boolean ("show-all", "Show All",
           "Show all devices, even those from hidden providers",
@@ -505,14 +510,13 @@ again:
   while (pending) {
     GstDeviceProvider *provider = pending->data;
 
-    if (gst_device_provider_can_monitor (provider)) {
-      GST_OBJECT_UNLOCK (monitor);
+    GST_OBJECT_UNLOCK (monitor);
 
-      if (!gst_device_provider_start (provider))
-        goto start_failed;
+    if (!gst_device_provider_start (provider))
+      goto start_failed;
+
+    GST_OBJECT_LOCK (monitor);
 
-      GST_OBJECT_LOCK (monitor);
-    }
     started = g_list_prepend (started, provider);
     pending = g_list_delete_link (pending, pending);
 
@@ -574,8 +578,7 @@ gst_device_monitor_stop (GstDeviceMonitor * monitor)
   while (started) {
     GstDeviceProvider *provider = started->data;
 
-    if (gst_device_provider_can_monitor (provider))
-      gst_device_provider_stop (provider);
+    gst_device_provider_stop (provider);
 
     started = g_list_delete_link (started, started);
     gst_object_unref (provider);
@@ -895,7 +898,7 @@ gst_device_monitor_set_show_all_devices (GstDeviceMonitor * monitor,
  * gst_device_monitor_get_show_all_devices:
  * @monitor: a #GstDeviceMonitor
  *
- * Get if @monitor is curretly showing all devices, even those from hidden
+ * Get if @monitor is currently showing all devices, even those from hidden
  * providers.
  *
  * Returns: %TRUE when all devices will be shown.
