@@ -50,6 +50,7 @@
 #include "gstmeta.h"
 #include "gstinfo.h"
 #include "gstutils.h"
+#include "gstquark.h"
 
 static GHashTable *metainfo = NULL;
 static GRWLock lock;
@@ -98,8 +99,7 @@ gst_meta_api_type_register (const gchar * api, const gchar ** tags)
     }
   }
 
-  g_type_set_qdata (type, g_quark_from_string ("tags"),
-      g_strdupv ((gchar **) tags));
+  g_type_set_qdata (type, GST_QUARK (TAGS), g_strdupv ((gchar **) tags));
 
   return type;
 }
@@ -136,7 +136,7 @@ gst_meta_api_type_get_tags (GType api)
   const gchar **tags;
   g_return_val_if_fail (api != 0, FALSE);
 
-  tags = g_type_get_qdata (api, g_quark_from_string ("tags"));
+  tags = g_type_get_qdata (api, GST_QUARK (TAGS));
 
   if (!tags[0])
     return NULL;
