@@ -622,7 +622,7 @@ gst_preset_default_get_property_names (GstPreset * preset)
         g_free (props);
       }
 
-      g_object_unref (child);
+      gst_object_unref (child);
     }
   }
   if (!result) {
