@@ -147,6 +147,19 @@ static GstStaticPadTemplate hsinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
+typedef struct
+{
+  GType api;
+  GstStructure *params;
+} ProposeMeta;
+
+static void
+propose_meta_clear (ProposeMeta * meta)
+{
+  if (meta->params)
+    gst_structure_free (meta->params);
+}
+
 struct _GstHarnessPrivate
 {
   gchar *element_sinkpad_name;
@@ -179,11 +192,17 @@ struct _GstHarnessPrivate
   GstAllocator *propose_allocator;
   GstAllocationParams propose_allocation_params;
 
+  GArray *propose_allocation_metas;
+
   gboolean blocking_push_mode;
   GCond blocking_push_cond;
   GMutex blocking_push_mutex;
   GMutex priv_mutex;
 
+  GCond buf_or_eos_cond;
+  GMutex buf_or_eos_mutex;
+  gboolean eos_received;
+
   GPtrArray *stress;
 };
 
@@ -197,10 +216,15 @@ gst_harness_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   g_mutex_lock (&priv->blocking_push_mutex);
   g_atomic_int_inc (&priv->recv_buffers);
 
-  if (priv->drop_buffers)
+  if (priv->drop_buffers) {
     gst_buffer_unref (buffer);
-  else
+  } else {
+    g_mutex_lock (&priv->buf_or_eos_mutex);
     g_async_queue_push (priv->buffer_queue, buffer);
+    g_cond_signal (&priv->buf_or_eos_cond);
+    g_mutex_unlock (&priv->buf_or_eos_mutex);
+  }
+
 
   if (priv->blocking_push_mode) {
     g_cond_wait (&priv->blocking_push_cond, &priv->blocking_push_mutex);
@@ -253,6 +277,12 @@ gst_harness_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     gst_object_unref (fwdpad);
     HARNESS_LOCK (h);
   } else {
+    if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+      g_mutex_lock (&priv->buf_or_eos_mutex);
+      priv->eos_received = TRUE;
+      g_cond_signal (&priv->buf_or_eos_cond);
+      g_mutex_unlock (&priv->buf_or_eos_mutex);
+    }
     g_async_queue_push (priv->sink_event_queue, event);
   }
   HARNESS_UNLOCK (h);
@@ -390,12 +420,34 @@ gst_harness_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
         gst_query_add_allocation_param (query,
             priv->propose_allocator, &priv->propose_allocation_params);
 
+        if (priv->propose_allocation_metas) {
+          guint i;
+          for (i = 0; i < priv->propose_allocation_metas->len; i++) {
+            ProposeMeta *meta =
+                &g_array_index (priv->propose_allocation_metas, ProposeMeta, i);
+            gst_query_add_allocation_meta (query, meta->api, meta->params);
+          }
+        }
+
         GST_DEBUG_OBJECT (pad, "proposing allocation %" GST_PTR_FORMAT,
             priv->propose_allocator);
       }
       HARNESS_UNLOCK (h);
       break;
     }
+    case GST_QUERY_CONTEXT:
+      HARNESS_LOCK (h);
+      if (priv->forwarding && priv->sink_forward_pad != NULL) {
+        GstPad *peer = gst_pad_get_peer (priv->sink_forward_pad);
+        g_assert (peer != NULL);
+        HARNESS_UNLOCK (h);
+        res = gst_pad_query (peer, query);
+        gst_object_unref (peer);
+      } else {
+        HARNESS_UNLOCK (h);
+        res = gst_pad_query_default (pad, parent, query);
+      }
+      break;
     default:
       res = gst_pad_query_default (pad, parent, query);
   }
@@ -516,13 +568,9 @@ static void
 gst_harness_setup_src_pad (GstHarness * h,
     GstStaticPadTemplate * src_tmpl, const gchar * element_sinkpad_name)
 {
-  GstHarnessPrivate *priv = h->priv;
   g_assert (src_tmpl);
   g_assert (h->srcpad == NULL);
 
-  priv->src_event_queue =
-      g_async_queue_new_full ((GDestroyNotify) gst_event_unref);
-
   /* sending pad */
   h->srcpad = gst_pad_new_from_static_template (src_tmpl, "src");
   g_assert (h->srcpad);
@@ -541,15 +589,9 @@ static void
 gst_harness_setup_sink_pad (GstHarness * h,
     GstStaticPadTemplate * sink_tmpl, const gchar * element_srcpad_name)
 {
-  GstHarnessPrivate *priv = h->priv;
   g_assert (sink_tmpl);
   g_assert (h->sinkpad == NULL);
 
-  priv->buffer_queue = g_async_queue_new_full (
-      (GDestroyNotify) gst_buffer_unref);
-  priv->sink_event_queue = g_async_queue_new_full (
-      (GDestroyNotify) gst_event_unref);
-
   /* receiving pad */
   h->sinkpad = gst_pad_new_from_static_template (sink_tmpl, "sink");
   g_assert (h->sinkpad);
@@ -637,13 +679,20 @@ gst_harness_new_empty (void)
   h->priv = g_new0 (GstHarnessPrivate, 1);
   priv = h->priv;
 
-  GST_DEBUG_OBJECT (h, "about to create new harness %p", h);
+  GST_DEBUG ("about to create new harness %p", h);
   priv->last_push_ts = GST_CLOCK_TIME_NONE;
   priv->latency_min = 0;
   priv->latency_max = GST_CLOCK_TIME_NONE;
   priv->drop_buffers = FALSE;
   priv->testclock = GST_TEST_CLOCK_CAST (gst_test_clock_new ());
 
+  priv->buffer_queue = g_async_queue_new_full (
+      (GDestroyNotify) gst_buffer_unref);
+  priv->src_event_queue = g_async_queue_new_full (
+      (GDestroyNotify) gst_event_unref);
+  priv->sink_event_queue = g_async_queue_new_full (
+      (GDestroyNotify) gst_event_unref);
+
   priv->propose_allocator = NULL;
   gst_allocation_params_init (&priv->propose_allocation_params);
 
@@ -651,6 +700,10 @@ gst_harness_new_empty (void)
   g_cond_init (&priv->blocking_push_cond);
   g_mutex_init (&priv->priv_mutex);
 
+  g_mutex_init (&priv->buf_or_eos_mutex);
+  g_cond_init (&priv->buf_or_eos_cond);
+  priv->eos_received = FALSE;
+
   priv->stress = g_ptr_array_new_with_free_func (
       (GDestroyNotify) gst_harness_stress_free);
 
@@ -736,7 +789,7 @@ gst_harness_add_element_full (GstHarness * h, GstElement * element,
 
   gst_harness_element_ref (h);
 
-  GST_DEBUG_OBJECT (h, "added element to harness %p "
+  GST_DEBUG ("added element to harness %p "
       "with element_srcpad_name (%p, %s, %s) and element_sinkpad_name (%p, %s, %s)",
       h, h->srcpad, GST_DEBUG_PAD_NAME (h->srcpad),
       h->sinkpad, GST_DEBUG_PAD_NAME (h->sinkpad));
@@ -1010,20 +1063,17 @@ gst_harness_teardown (GstHarness * h)
     g_mutex_unlock (&priv->blocking_push_mutex);
   }
 
-  if (h->src_harness) {
+  if (h->src_harness)
     gst_harness_teardown (h->src_harness);
-  }
+  h->src_harness = NULL;
 
+  HARNESS_LOCK (h);
   gst_object_replace ((GstObject **) & priv->sink_forward_pad, NULL);
-  if (h->sink_harness) {
+  HARNESS_UNLOCK (h);
+
+  if (h->sink_harness)
     gst_harness_teardown (h->sink_harness);
-  }
-
-  if (priv->src_caps)
-    gst_caps_unref (priv->src_caps);
-
-  if (priv->sink_caps)
-    gst_caps_unref (priv->sink_caps);
+  h->sink_harness = NULL;
 
   if (h->srcpad) {
     if (gst_pad_is_request_pad (GST_PAD_PEER (h->srcpad)))
@@ -1031,10 +1081,17 @@ gst_harness_teardown (GstHarness * h)
     g_free (priv->element_sinkpad_name);
 
     gst_pad_set_active (h->srcpad, FALSE);
-    gst_object_unref (h->srcpad);
 
-    g_async_queue_unref (priv->src_event_queue);
+    /* Make sure our funcs are not called after harness is teared down since
+     * they try to access this harness through pad data */
+    GST_PAD_STREAM_LOCK (h->srcpad);
+    gst_pad_set_event_function (h->srcpad, NULL);
+    gst_pad_set_query_function (h->srcpad, NULL);
+    GST_PAD_STREAM_UNLOCK (h->srcpad);
+
+    gst_object_unref (h->srcpad);
   }
+  h->srcpad = NULL;
 
   if (h->sinkpad) {
     if (gst_pad_is_request_pad (GST_PAD_PEER (h->sinkpad)))
@@ -1042,18 +1099,37 @@ gst_harness_teardown (GstHarness * h)
     g_free (priv->element_srcpad_name);
 
     gst_pad_set_active (h->sinkpad, FALSE);
-    gst_object_unref (h->sinkpad);
 
-    g_async_queue_unref (priv->buffer_queue);
-    g_async_queue_unref (priv->sink_event_queue);
+    /* Make sure our funcs are not called after harness is teared down since
+     * they try to access this harness through pad data */
+    GST_PAD_STREAM_LOCK (h->sinkpad);
+    gst_pad_set_chain_function (h->sinkpad, NULL);
+    gst_pad_set_event_function (h->sinkpad, NULL);
+    gst_pad_set_query_function (h->sinkpad, NULL);
+    GST_PAD_STREAM_UNLOCK (h->sinkpad);
+
+    gst_object_unref (h->sinkpad);
   }
+  h->sinkpad = NULL;
+
+  if (priv->src_caps)
+    gst_caps_unref (priv->src_caps);
+  priv->src_caps = NULL;
+
+  if (priv->sink_caps)
+    gst_caps_unref (priv->sink_caps);
+  priv->sink_caps = NULL;
 
   gst_object_replace ((GstObject **) & priv->propose_allocator, NULL);
   gst_object_replace ((GstObject **) & priv->allocator, NULL);
   gst_object_replace ((GstObject **) & priv->pool, NULL);
 
+  if (priv->propose_allocation_metas)
+    g_array_unref (priv->propose_allocation_metas);
+  priv->propose_allocation_metas = NULL;
+
   /* if we hold the last ref, set to NULL */
-  if (gst_harness_element_unref (h) == 0) {
+  if (h->element != NULL && gst_harness_element_unref (h) == 0) {
     gboolean state_change;
     GstState state, pending;
     state_change = gst_element_set_state (h->element, GST_STATE_NULL);
@@ -1067,13 +1143,29 @@ gst_harness_teardown (GstHarness * h)
   g_mutex_clear (&priv->blocking_push_mutex);
   g_mutex_clear (&priv->priv_mutex);
 
-  g_ptr_array_unref (priv->stress);
+  g_mutex_clear (&priv->buf_or_eos_mutex);
+  g_cond_clear (&priv->buf_or_eos_cond);
+  priv->eos_received = FALSE;
 
-  gst_object_unref (h->element);
+  g_async_queue_unref (priv->buffer_queue);
+  priv->buffer_queue = NULL;
+  g_async_queue_unref (priv->src_event_queue);
+  priv->src_event_queue = NULL;
+  g_async_queue_unref (priv->sink_event_queue);
+  priv->sink_event_queue = NULL;
+
+  g_ptr_array_unref (priv->stress);
+  priv->stress = NULL;
+
+  if (h->element) {
+    gst_object_unref (h->element);
+    h->element = NULL;
+  }
 
   gst_object_replace ((GstObject **) & priv->testclock, NULL);
 
   g_free (h->priv);
+  h->priv = NULL;
   g_free (h);
 }
 
@@ -1082,7 +1174,7 @@ gst_harness_teardown (GstHarness * h)
  * @h: a #GstHarness
  * @srcpad: a #GstPad to link to the harness sinkpad
  *
- * Links the specifed #GstPad the @GstHarness sinkpad. This can be useful if
+ * Links the specified #GstPad the @GstHarness sinkpad. This can be useful if
  * perhaps the srcpad did not exist at the time of creating the harness,
  * like a demuxer that provides a sometimes-pad after receiving data.
  *
@@ -1108,7 +1200,7 @@ gst_harness_add_element_src_pad (GstHarness * h, GstPad * srcpad)
  * @h: a #GstHarness
  * @sinkpad: a #GstPad to link to the harness srcpad
  *
- * Links the specifed #GstPad the @GstHarness srcpad.
+ * Links the specified #GstPad the @GstHarness srcpad.
  *
  * MT safe.
  *
@@ -1152,6 +1244,7 @@ gst_harness_set_src_caps (GstHarness * h, GstCaps * caps)
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   handled = gst_pad_push_event (h->srcpad, gst_event_new_segment (&segment));
+  g_assert (handled);
 }
 
 /**
@@ -1337,30 +1430,15 @@ gst_harness_set_time (GstHarness * h, GstClockTime time)
  * MT safe.
  *
  * Returns: a @gboolean %TRUE if the waits have been registered, %FALSE if not.
- * (Could be that it timed out waiting or that more waits then waits was found)
+ * (Could be that it timed out waiting or that more waits than waits was found)
  *
  * Since: 1.6
  */
 gboolean
 gst_harness_wait_for_clock_id_waits (GstHarness * h, guint waits, guint timeout)
 {
-  GstTestClock *testclock = h->priv->testclock;
-  gint64 start_time;
-  gboolean ret;
-
-  start_time = g_get_monotonic_time ();
-  while (gst_test_clock_peek_id_count (testclock) < waits) {
-    gint64 time_spent;
-
-    g_usleep (G_USEC_PER_SEC / 1000);
-    time_spent = g_get_monotonic_time () - start_time;
-    if ((time_spent / G_USEC_PER_SEC) > timeout)
-      break;
-  }
-
-  ret = (waits == gst_test_clock_peek_id_count (testclock));
-
-  return ret;
+  return gst_test_clock_timed_wait_for_multiple_pending_ids (h->priv->testclock,
+      waits, timeout * 1000, NULL);
 }
 
 /**
@@ -1373,7 +1451,7 @@ gst_harness_wait_for_clock_id_waits (GstHarness * h, guint waits, guint timeout)
  * 3: Release the #GstClockID wait.
  * Together, this provides an easy way to not have to think about the details
  * around clocks and time, but still being able to write deterministic tests
- * that are dependant on this. A "crank" can be though of as the notion of
+ * that are dependent on this. A "crank" can be though of as the notion of
  * manually driving the clock forward to its next logical step.
  *
  * MT safe.
@@ -1427,7 +1505,7 @@ gst_harness_crank_multiple_clock_waits (GstHarness * h, guint waits)
  *
  * This will set the harnessed #GstElement to %GST_STATE_PLAYING.
  * #GstElements without a sink-#GstPad and with the %GST_ELEMENT_FLAG_SOURCE
- * flag set is concidered a src #GstElement
+ * flag set is considered a src #GstElement
  * Non-src #GstElements (like sinks and filters) are automatically set to
  * playing by the #GstHarness, but src #GstElements are not to avoid them
  * starting to produce buffers.
@@ -1504,13 +1582,14 @@ gst_harness_set_forwarding (GstHarness * h, gboolean forwarding)
     gst_harness_set_forwarding (h->sink_harness, forwarding);
 }
 
+/*
+* Call with HARNESS_LOCK
+*/
 static void
 gst_harness_set_forward_pad (GstHarness * h, GstPad * fwdpad)
 {
-  HARNESS_LOCK (h);
   gst_object_replace ((GstObject **) & h->priv->sink_forward_pad,
       (GstObject *) fwdpad);
-  HARNESS_UNLOCK (h);
 }
 
 /**
@@ -1541,8 +1620,8 @@ gst_harness_create_buffer (GstHarness * h, gsize size)
     flow = gst_buffer_pool_acquire_buffer (priv->pool, &ret, NULL);
     g_assert_cmpint (flow, ==, GST_FLOW_OK);
     if (gst_buffer_get_size (ret) != size) {
-      GST_DEBUG_OBJECT (h,
-          "use fallback, pool is configured with a different size (%zu != %zu)",
+      GST_DEBUG ("use fallback, pool is configured with a different size (%"
+          G_GSIZE_FORMAT " != %" G_GSIZE_FORMAT ")",
           size, gst_buffer_get_size (ret));
       gst_buffer_unref (ret);
       ret = NULL;
@@ -1611,6 +1690,40 @@ gst_harness_pull (GstHarness * h)
   return buf;
 }
 
+/**
+ * gst_harness_pull_until_eos:
+ * @h: a #GstHarness
+ * @buf: (out) (transfer full): A #GstBuffer, or %NULL if EOS or timeout occures
+ *   first.
+ *
+ * Pulls a #GstBuffer from the #GAsyncQueue on the #GstHarness sinkpad. The pull
+ * will block until an EOS event is received, or timeout in 60 seconds.
+ * MT safe.
+ *
+ * Returns: %TRUE on success, %FALSE on timeout.
+ *
+ * Since: 1.18
+ */
+gboolean
+gst_harness_pull_until_eos (GstHarness * h, GstBuffer ** buf)
+{
+  GstHarnessPrivate *priv = h->priv;
+  gboolean success = TRUE;
+  gint64 end_time = g_get_monotonic_time () + 60 * G_TIME_SPAN_SECOND;
+
+  g_mutex_lock (&priv->buf_or_eos_mutex);
+  while (success) {
+    *buf = g_async_queue_try_pop (priv->buffer_queue);
+    if (*buf || priv->eos_received)
+      break;
+    success = g_cond_wait_until (&priv->buf_or_eos_cond,
+        &priv->buf_or_eos_mutex, end_time);
+  }
+  g_mutex_unlock (&priv->buf_or_eos_mutex);
+
+  return success;
+}
+
 /**
  * gst_harness_try_pull:
  * @h: a #GstHarness
@@ -2161,6 +2274,38 @@ gst_harness_set_propose_allocator (GstHarness * h, GstAllocator * allocator,
     priv->propose_allocation_params = *params;
 }
 
+/**
+ * gst_harness_add_propose_allocation_meta:
+ * @h: a #GstHarness
+ * @api: a metadata API
+ * @params: (allow-none) (transfer none): API specific parameters
+ *
+ * Add api with params as one of the supported metadata API to propose when
+ * receiving an allocation query.
+ *
+ * MT safe.
+ *
+ * Since: 1.16
+ */
+void
+gst_harness_add_propose_allocation_meta (GstHarness * h, GType api,
+    const GstStructure * params)
+{
+  GstHarnessPrivate *priv = h->priv;
+  ProposeMeta meta;
+
+  meta.api = api;
+  meta.params = params ? gst_structure_copy (params) : NULL;
+
+  if (!priv->propose_allocation_metas) {
+    priv->propose_allocation_metas =
+        g_array_new (FALSE, FALSE, sizeof (ProposeMeta));
+    g_array_set_clear_func (priv->propose_allocation_metas,
+        (GDestroyNotify) propose_meta_clear);
+  }
+  g_array_append_val (priv->propose_allocation_metas, meta);
+}
+
 /**
  * gst_harness_add_src_harness:
  * @h: a #GstHarness
@@ -2172,7 +2317,7 @@ gst_harness_set_propose_allocator (GstHarness * h, GstAllocator * allocator,
  * By adding a src-type #GstElement, it is then easy to use functions like
  * gst_harness_push_from_src or gst_harness_src_crank_and_push_many
  * to provide your harnessed element with input. The @has_clock_wait variable
- * is a greate way to control you src-element with, in that you can have it
+ * is a great way to control you src-element with, in that you can have it
  * produce a buffer for you by simply cranking the clock, and not have it
  * spin out of control producing buffers as fast as possible.
  *
@@ -2189,7 +2334,11 @@ gst_harness_add_src_harness (GstHarness * h,
   if (h->src_harness)
     gst_harness_teardown (h->src_harness);
   h->src_harness = src_harness;
+
+  HARNESS_LOCK (h->src_harness);
   gst_harness_set_forward_pad (h->src_harness, h->srcpad);
+  HARNESS_UNLOCK (h->src_harness);
+
   h->src_harness->priv->has_clock_wait = has_clock_wait;
   gst_harness_set_forwarding (h->src_harness, h->priv->forwarding);
 }
@@ -2350,8 +2499,8 @@ gst_harness_src_push_event (GstHarness * h)
 static gboolean
 forward_sticky_events (GstPad * pad, GstEvent ** ev, gpointer user_data)
 {
-  GstHarness *h = user_data;
-  return gst_pad_push_event (h->priv->sink_forward_pad, gst_event_ref (*ev));
+  GstPad *fwdpad = user_data;
+  return gst_pad_push_event (fwdpad, gst_event_ref (*ev));
 }
 
 /**
@@ -2364,7 +2513,7 @@ forward_sticky_events (GstPad * pad, GstEvent ** ev, gpointer user_data)
  * responses the element output might create in sink elements. An example might
  * be an existing sink providing some analytical data on the input it receives that
  * can be useful to your testing. If the goal is to test a sink-element itself,
- * this is better acheived using gst_harness_new directly on the sink.
+ * this is better achieved using gst_harness_new directly on the sink.
  *
  * If a sink-harness already exists it will be replaced.
  *
@@ -2375,17 +2524,35 @@ forward_sticky_events (GstPad * pad, GstEvent ** ev, gpointer user_data)
 void
 gst_harness_add_sink_harness (GstHarness * h, GstHarness * sink_harness)
 {
-  GstHarnessPrivate *priv = h->priv;
+  GstHarnessPrivate *priv;
+  GstPad *fwdpad;
+
+  HARNESS_LOCK (h);
+  priv = h->priv;
 
   if (h->sink_harness) {
     gst_harness_set_forward_pad (h, NULL);
     gst_harness_teardown (h->sink_harness);
   }
   h->sink_harness = sink_harness;
-  gst_harness_set_forward_pad (h, h->sink_harness->srcpad);
-  if (priv->forwarding && h->sinkpad)
-    gst_pad_sticky_events_foreach (h->sinkpad, forward_sticky_events, h);
+
+  fwdpad = h->sink_harness->srcpad;
+  if (fwdpad)
+    gst_object_ref (fwdpad);
+
+  if (priv->forwarding && h->sinkpad && fwdpad) {
+    HARNESS_UNLOCK (h);
+    gst_pad_sticky_events_foreach (h->sinkpad, forward_sticky_events, fwdpad);
+    HARNESS_LOCK (h);
+  }
+
+  gst_harness_set_forward_pad (h, fwdpad);
+  if (fwdpad)
+    gst_object_unref (fwdpad);
+
   gst_harness_set_forwarding (h->sink_harness, priv->forwarding);
+
+  HARNESS_UNLOCK (h);
 }
 
 /**
@@ -2500,6 +2667,18 @@ gst_harness_find_element (GstHarness * h, const gchar * element_name)
   GstIterator *iter;
   GValue data = G_VALUE_INIT;
 
+  if (!GST_IS_BIN (h->element)) {
+    GstPluginFeature *feature;
+
+    g_return_val_if_fail (GST_IS_ELEMENT (h->element), NULL);
+
+    feature = GST_PLUGIN_FEATURE (gst_element_get_factory (h->element));
+    if (!strcmp (element_name, gst_plugin_feature_get_name (feature)))
+      return gst_object_ref (h->element);
+
+    return NULL;
+  }
+
   iter = gst_bin_iterate_elements (GST_BIN (h->element));
   done = FALSE;
 
