@@ -21,12 +21,19 @@
  */
 /**
  * SECTION:gstdirectcontrolbinding
+ * @title: GstDirectControlBinding
  * @short_description: direct attachment for control sources
  *
  * A value mapping object that attaches control sources to gobject properties. It
- * will map the control values [0.0 ... 1.0] to the target property range. If a
- * control value is outside of the range, it will be clipped.
+ * will map the control values directly to the target property range. If a
+ * non-absolute direct control binding is used, the value range [0.0 ... 1.0]
+ * is mapped to full target property range, and all values outside the range
+ * will be clipped. An absolute control binding will not do any value
+ * transformations.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <glib-object.h>
 #include <gst/gst.h>
@@ -208,6 +215,7 @@ gst_direct_control_binding_class_init (GstDirectControlBindingClass * klass)
 static void
 gst_direct_control_binding_init (GstDirectControlBinding * self)
 {
+  self->last_value = G_MAXDOUBLE;
 }
 
 static GObject *
@@ -420,7 +428,7 @@ gst_direct_control_binding_get_value_array (GstControlBinding * _self,
     gpointer values_)
 {
   GstDirectControlBinding *self = GST_DIRECT_CONTROL_BINDING (_self);
-  gint i;
+  guint i;
   gdouble *src_val;
   gboolean res = FALSE;
   GstDirectControlBindingConvertValue convert;
@@ -463,7 +471,7 @@ gst_direct_control_binding_get_g_value_array (GstControlBinding * _self,
     GValue * values)
 {
   GstDirectControlBinding *self = GST_DIRECT_CONTROL_BINDING (_self);
-  gint i;
+  guint i;
   gdouble *src_val;
   gboolean res = FALSE;
   GType type;
@@ -508,7 +516,8 @@ gst_direct_control_binding_get_g_value_array (GstControlBinding * _self,
  * @cs: the control source
  *
  * Create a new control-binding that attaches the #GstControlSource to the
- * #GObject property.
+ * #GObject property. It will map the control source range [0.0 ... 1.0] to
+ * the full target property range, and clip all values outside this range.
  *
  * Returns: (transfer floating): the new #GstDirectControlBinding
  */
@@ -527,7 +536,8 @@ gst_direct_control_binding_new (GstObject * object, const gchar * property_name,
  * @cs: the control source
  *
  * Create a new control-binding that attaches the #GstControlSource to the
- * #GObject property.
+ * #GObject property. It will directly map the control source values to the
+ * target property range without any transformations.
  *
  * Returns: (transfer floating): the new #GstDirectControlBinding
  *
