@@ -24,6 +24,7 @@
 
 /**
  * SECTION:gsttimedvaluecontrolsource
+ * @title: GstTimedValueControlSource
  * @short_description: timed value control source base class
  *
  * Base class for #GstControlSource that use time-stamped values.
@@ -34,6 +35,9 @@
  * All functions are MT-safe.
  *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <glib-object.h>
 #include <gst/gst.h>
@@ -63,14 +67,13 @@ enum
 
 static guint gst_timed_value_control_source_signals[LAST_SIGNAL] = { 0 };
 
-/*
+/**
  * gst_control_point_free:
- * @prop: the object to free
+ * @cp: the object to free
  *
- * Private method which frees all data allocated by a #GstControlPoint
- * instance.
+ * Frees all data allocated by a #GstControlPoint instance.
  */
-static void
+void
 gst_control_point_free (GstControlPoint * cp)
 {
   g_return_if_fail (cp);
@@ -78,10 +81,18 @@ gst_control_point_free (GstControlPoint * cp)
   g_slice_free (GstControlPoint, cp);
 }
 
-static gpointer
-gst_control_point_copy (GstControlPoint * boxed)
+/**
+ * gst_control_point_copy:
+ * @cp: The control point to copy
+ *
+ * Copies a #GstControlPoint
+ *
+ * Returns: A copy of @cp
+ */
+GstControlPoint *
+gst_control_point_copy (GstControlPoint * cp)
 {
-  return g_slice_dup (GstControlPoint, boxed);
+  return g_slice_dup (GstControlPoint, cp);
 }
 
 GType
@@ -463,15 +474,14 @@ gst_timed_value_control_source_class_init (GstTimedValueControlSourceClass
    * @self: The #GstTimedValueControlSource on which a #GstTimedValue has changed
    * @timed_value: The #GstTimedValue where the value changed
    *
-   * Emited right after the new value has been set on @timed_signals
+   * Emitted right after the new value has been set on @timed_signals
    *
    * Since: 1.6
    */
   gst_timed_value_control_source_signals[VALUE_CHANGED_SIGNAL] =
       g_signal_new ("value-changed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, 0, NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      gst_control_point_get_type ());
+      NULL, NULL, G_TYPE_NONE, 1, gst_control_point_get_type ());
 
   /**
    * GstTimedValueControlSource::value-added
@@ -479,15 +489,14 @@ gst_timed_value_control_source_class_init (GstTimedValueControlSourceClass
    *        added
    * @timed_value: The newly added #GstTimedValue
    *
-   * Emited right after the new value has been added to @self
+   * Emitted right after the new value has been added to @self
    *
    * Since: 1.6
    */
   gst_timed_value_control_source_signals[VALUE_ADDED_SIGNAL] =
       g_signal_new ("value-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, 0, NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      gst_control_point_get_type ());
+      NULL, NULL, G_TYPE_NONE, 1, gst_control_point_get_type ());
 
   /**
    * GstTimedValueControlSource::value-removed
@@ -495,15 +504,14 @@ gst_timed_value_control_source_class_init (GstTimedValueControlSourceClass
    *        removed
    * @timed_value: The removed #GstTimedValue
    *
-   * Emited when @timed_value is removed from @self
+   * Emitted when @timed_value is removed from @self
    *
    * Since: 1.6
    */
   gst_timed_value_control_source_signals[VALUE_REMOVED_SIGNAL] =
       g_signal_new ("value-removed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, 0, NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      gst_control_point_get_type ());
+      NULL, NULL, G_TYPE_NONE, 1, gst_control_point_get_type ());
 
 
   gobject_class->finalize = gst_timed_value_control_source_finalize;
