@@ -947,6 +947,8 @@ send_delay_req_timeout (PtpPendingSync * sync)
   header.type = TYPE_EVENT;
   header.size = 44;
 
+  GST_TRACE ("Sending delay_req to domain %u", sync->domain);
+
   gst_byte_writer_init_with_data (&writer, delay_req, 44, FALSE);
   gst_byte_writer_put_uint8_unchecked (&writer, PTP_MESSAGE_TYPE_DELAY_REQ);
   gst_byte_writer_put_uint8_unchecked (&writer, 2);
@@ -1022,8 +1024,10 @@ send_delay_req (PtpDomainData * domain, PtpPendingSync * sync)
   GSource *timeout_source;
 
   if (domain->last_delay_req != 0
-      && domain->last_delay_req + domain->min_delay_req_interval > now)
+      && domain->last_delay_req + domain->min_delay_req_interval > now) {
+    GST_TRACE ("Too soon to send new DELAY_REQ");
     return FALSE;
+  }
 
   domain->last_delay_req = now;
   sync->delay_req_seqnum = domain->last_delay_req_seqnum++;
@@ -1064,12 +1068,17 @@ update_ptp_time (PtpDomainData * domain, PtpPendingSync * sync)
   GstClockTime max_discont, estimated_ptp_time_min, estimated_ptp_time_max;
   gboolean now_synced;
 #endif
-
 #ifdef USE_ONLY_SYNC_WITH_DELAY
   GstClockTime mean_path_delay;
+#endif
 
-  if (sync->delay_req_send_time_local == GST_CLOCK_TIME_NONE)
+  GST_TRACE ("Updating PTP time");
+
+#ifdef USE_ONLY_SYNC_WITH_DELAY
+  if (sync->delay_req_send_time_local == GST_CLOCK_TIME_NONE) {
+    GST_TRACE ("Not updating - no delay_req sent");
     return;
+  }
 
   /* IEEE 1588 11.3 */
   mean_path_delay =
@@ -1439,8 +1448,10 @@ handle_sync_message (PtpMessage * msg, GstClockTime receive_time)
   PtpPendingSync *sync = NULL;
 
   /* Don't consider messages with the alternate master flag set */
-  if ((msg->flag_field & 0x0100))
+  if ((msg->flag_field & 0x0100)) {
+    GST_TRACE ("Ignoring sync message with alternate-master flag");
     return;
+  }
 
   for (l = domain_data; l; l = l->next) {
     PtpDomainData *tmp = l->data;
@@ -1520,6 +1531,7 @@ handle_sync_message (PtpMessage * msg, GstClockTime receive_time)
 
   if ((msg->flag_field & 0x0200)) {
     /* Wait for FOLLOW_UP */
+    GST_TRACE ("Waiting for FOLLOW_UP msg");
   } else {
     sync->sync_send_time_remote =
         PTP_TIMESTAMP_TO_GST_CLOCK_TIME (msg->message_specific.
@@ -1557,9 +1569,13 @@ handle_follow_up_message (PtpMessage * msg, GstClockTime receive_time)
   PtpDomainData *domain = NULL;
   PtpPendingSync *sync = NULL;
 
+  GST_TRACE ("Processing FOLLOW_UP message");
+
   /* Don't consider messages with the alternate master flag set */
-  if ((msg->flag_field & 0x0100))
+  if ((msg->flag_field & 0x0100)) {
+    GST_TRACE ("Ignoring FOLLOW_UP with alternate-master flag");
     return;
+  }
 
   for (l = domain_data; l; l = l->next) {
     PtpDomainData *tmp = l->data;
@@ -1570,14 +1586,18 @@ handle_follow_up_message (PtpMessage * msg, GstClockTime receive_time)
     }
   }
 
-  if (!domain)
+  if (!domain) {
+    GST_TRACE ("No domain match for FOLLOW_UP msg");
     return;
+  }
 
   /* If we have a master clock, ignore this message if it's not coming from there */
   if (domain->have_master_clock
       && compare_clock_identity (&domain->master_clock_identity,
-          &msg->source_port_identity) != 0)
+          &msg->source_port_identity) != 0) {
+    GST_TRACE ("FOLLOW_UP msg not from current clock master. Ignoring");
     return;
+  }
 
   /* Check if we know about this one */
   for (l = domain->pending_syncs.head; l; l = l->next) {
@@ -1589,12 +1609,16 @@ handle_follow_up_message (PtpMessage * msg, GstClockTime receive_time)
     }
   }
 
-  if (!sync)
+  if (!sync) {
+    GST_TRACE ("Ignoring FOLLOW_UP with no pending SYNC");
     return;
+  }
 
   /* Got a FOLLOW_UP for this already */
-  if (sync->sync_send_time_remote != GST_CLOCK_TIME_NONE)
+  if (sync->sync_send_time_remote != GST_CLOCK_TIME_NONE) {
+    GST_TRACE ("Got repeat FOLLOW_UP. Ignoring");
     return;
+  }
 
   if (sync->sync_recv_time_local >= receive_time) {
     GST_ERROR ("Got bogus follow up in domain %u: %" GST_TIME_FORMAT " > %"
@@ -1729,9 +1753,13 @@ handle_ptp_message (PtpMessage * msg, GstClockTime receive_time)
 {
   /* Ignore our own messages */
   if (msg->source_port_identity.clock_identity == ptp_clock_id.clock_identity &&
-      msg->source_port_identity.port_number == ptp_clock_id.port_number)
+      msg->source_port_identity.port_number == ptp_clock_id.port_number) {
+    GST_TRACE ("Ignoring our own message");
     return;
+  }
 
+  GST_TRACE ("Message type %d receive_time %" GST_TIME_FORMAT,
+      msg->message_type, GST_TIME_ARGS (receive_time));
   switch (msg->message_type) {
     case PTP_MESSAGE_TYPE_ANNOUNCE:
       handle_announce_message (msg, receive_time);
@@ -1984,7 +2012,7 @@ gst_ptp_is_supported (void)
  *
  * Check if the GStreamer PTP clock subsystem is initialized.
  *
- * Returns: %TRUE if the GStreamer PTP clock subsystem is intialized.
+ * Returns: %TRUE if the GStreamer PTP clock subsystem is initialized.
  *
  * Since: 1.6
  */
@@ -2294,9 +2322,6 @@ enum
   PROP_GRANDMASTER_CLOCK_ID
 };
 
-#define GST_PTP_CLOCK_GET_PRIVATE(obj)  \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_PTP_CLOCK, GstPtpClockPrivate))
-
 struct _GstPtpClockPrivate
 {
   guint domain;
@@ -2305,7 +2330,7 @@ struct _GstPtpClockPrivate
 };
 
 #define gst_ptp_clock_parent_class parent_class
-G_DEFINE_TYPE (GstPtpClock, gst_ptp_clock, GST_TYPE_SYSTEM_CLOCK);
+G_DEFINE_TYPE_WITH_PRIVATE (GstPtpClock, gst_ptp_clock, GST_TYPE_SYSTEM_CLOCK);
 
 static void gst_ptp_clock_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -2324,8 +2349,6 @@ gst_ptp_clock_class_init (GstPtpClockClass * klass)
   gobject_class = G_OBJECT_CLASS (klass);
   clock_class = GST_CLOCK_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GstPtpClockPrivate));
-
   gobject_class->finalize = gst_ptp_clock_finalize;
   gobject_class->get_property = gst_ptp_clock_get_property;
   gobject_class->set_property = gst_ptp_clock_set_property;
@@ -2359,7 +2382,7 @@ gst_ptp_clock_init (GstPtpClock * self)
 {
   GstPtpClockPrivate *priv;
 
-  self->priv = priv = GST_PTP_CLOCK_GET_PRIVATE (self);
+  self->priv = priv = gst_ptp_clock_get_instance_private (self);
 
   GST_OBJECT_FLAG_SET (self, GST_CLOCK_FLAG_CAN_SET_MASTER);
   GST_OBJECT_FLAG_SET (self, GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC);
