@@ -23,12 +23,12 @@
  * @title: concat
  * @see_also: #GstFunnel
  *
- * Concatenates streams together to one continous stream.
+ * Concatenates streams together to one continuous stream.
  *
  * All streams but the current one are blocked until the current one
  * finished with %GST_EVENT_EOS. Then the next stream is enabled, while
- * keeping the running time continous for %GST_FORMAT_TIME segments or
- * keeping the segment continous for %GST_FORMAT_BYTES segments.
+ * keeping the running time continuous for %GST_FORMAT_TIME segments or
+ * keeping the segment continuous for %GST_FORMAT_BYTES segments.
  *
  * Streams are switched in the order in which the sinkpads were requested.
  *
@@ -324,6 +324,8 @@ gst_concat_request_new_pad (GstElement * element, GstPadTemplate * templ,
   if (do_notify)
     gst_concat_notify_active_pad (self);
 
+  GST_DEBUG_OBJECT (sinkpad, "requested pad");
+
   return sinkpad;
 }
 
@@ -337,7 +339,7 @@ gst_concat_release_pad (GstElement * element, GstPad * pad)
   gboolean eos = FALSE;
   gboolean do_notify = FALSE;
 
-  GST_DEBUG_OBJECT (self, "releasing pad");
+  GST_DEBUG_OBJECT (pad, "releasing pad");
 
   g_mutex_lock (&self->lock);
   spad->flushing = TRUE;
@@ -428,7 +430,7 @@ gst_concat_sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   GstConcat *self = GST_CONCAT (parent);
   GstConcatPad *spad = GST_CONCAT_PAD (pad);
 
-  GST_LOG_OBJECT (pad, "received buffer %p", buffer);
+  GST_LOG_OBJECT (pad, "received buffer %" GST_PTR_FORMAT, buffer);
 
   if (!gst_concat_pad_wait (spad, self))
     return GST_FLOW_FLUSHING;
@@ -453,7 +455,8 @@ gst_concat_sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
   ret = gst_pad_push (self->srcpad, buffer);
 
-  GST_LOG_OBJECT (pad, "handled buffer %s", gst_flow_get_name (ret));
+  GST_LOG_OBJECT (pad, "handled buffer %s, last_stop %" GST_TIME_FORMAT,
+      gst_flow_get_name (ret), GST_TIME_ARGS (self->last_stop));
 
   return ret;
 }
@@ -575,14 +578,30 @@ gst_concat_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
           /* We know no duration */
           segment.duration = -1;
 
-          /* Update segment values to be continous with last stream */
+          /* Update segment values to be continuous with last stream */
           if (self->format == GST_FORMAT_TIME) {
+            GST_DEBUG_OBJECT (self,
+                "Updating segment base %" GST_TIME_FORMAT " + %" GST_TIME_FORMAT
+                " = %" GST_TIME_FORMAT, GST_TIME_ARGS (segment.base),
+                GST_TIME_ARGS (self->current_start_offset),
+                GST_TIME_ARGS (segment.base + self->current_start_offset));
             segment.base += self->current_start_offset;
           } else {
             /* Shift start/stop byte position */
+            GST_DEBUG_OBJECT (self,
+                "Updating segment start %" G_GUINT64_FORMAT " + %"
+                G_GUINT64_FORMAT " = %" G_GUINT64_FORMAT, segment.start,
+                self->current_start_offset,
+                segment.start + self->current_start_offset);
             segment.start += self->current_start_offset;
-            if (segment.stop != -1)
+            if (segment.stop != -1) {
+              GST_DEBUG_OBJECT (self,
+                  "Updating segment stop %" G_GUINT64_FORMAT " + %"
+                  G_GUINT64_FORMAT " = %" G_GUINT64_FORMAT, segment.stop,
+                  self->current_start_offset,
+                  segment.stop + self->current_start_offset);
               segment.stop += self->current_start_offset;
+            }
           }
         }
         topush = gst_event_new_segment (&segment);
@@ -653,6 +672,7 @@ gst_concat_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
           GST_DEBUG_OBJECT (self,
               "resetting start offset to 0 after flushing with reset_time = TRUE");
           self->current_start_offset = 0;
+          self->last_stop = GST_CLOCK_TIME_NONE;
         }
         ret = gst_pad_event_default (pad, parent, event);
       } else {
