@@ -31,7 +31,7 @@
  * The above mentioned pipeline should dump data packets to the console.
  *
  * If the #GstFdSrc:timeout property is set to a value bigger than 0, fdsrc will
- * generate an element message named <classname>&quot;GstFdSrcTimeout&quot;</classname>
+ * generate an element message named `GstFdSrcTimeout`
  * if no data was received in the given timeout.
  *
  * The message's structure contains one field:
@@ -80,10 +80,14 @@
 
 #include "gstfdsrc.h"
 
+#define struct_stat struct stat
+
 #ifdef __BIONIC__               /* Android */
 #if defined(__ANDROID_API__) && __ANDROID_API__ >= 21
 #undef fstat
 #define fstat fstat64
+#undef struct_stat
+#define struct_stat struct stat64
 #endif
 #endif
 
@@ -208,7 +212,7 @@ gst_fd_src_dispose (GObject * obj)
 static void
 gst_fd_src_update_fd (GstFdSrc * src, guint64 size)
 {
-  struct stat stat_results;
+  struct_stat stat_results;
 
   GST_DEBUG_OBJECT (src, "fdset %p, old_fd %d, new_fd %d", src->fdset, src->fd,
       src->new_fd);
@@ -544,7 +548,7 @@ static gboolean
 gst_fd_src_get_size (GstBaseSrc * bsrc, guint64 * size)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
-  struct stat stat_results;
+  struct_stat stat_results;
 
   if (src->size != -1) {
     *size = src->size;
