@@ -19,10 +19,10 @@
  * Boston, MA 02110-1301, USA.
  */
 /**
- * SECTION:element-rusagetracer
+ * SECTION:tracer-rusage
  * @short_description: log resource usage stats
  *
- * A tracing module that take rusage() snapshots and logs them.
+ * A tracing module that take `rusage()` snapshots and logs them.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,11 +60,15 @@ static GstTracerRecord *tr_proc, *tr_thread;
 
 typedef struct
 {
-  /* time spend in this thread */
+  /* time spent in this thread */
   GstClockTime tthread;
   GstTraceValues *tvs_thread;
 } GstThreadStats;
 
+static void free_thread_stats (gpointer data);
+
+static GPrivate thread_stats_key = G_PRIVATE_INIT (free_thread_stats);
+
 /* data helper */
 
 static void
@@ -140,12 +144,13 @@ update_trace_value (GstTraceValues * self, GstClockTime nts,
   return ret;
 }
 
-
 static void
 free_thread_stats (gpointer data)
 {
-  free_trace_values (((GstThreadStats *) data)->tvs_thread);
-  g_slice_free (GstThreadStats, data);
+  GstThreadStats *stats = data;
+
+  free_trace_values (stats->tvs_thread);
+  g_free (stats);
 }
 
 static void
@@ -199,10 +204,10 @@ do_stats (GstTracer * obj, guint64 ts)
 #endif
 #endif
   /* get stats record for current thread */
-  if (!(stats = g_hash_table_lookup (self->threads, thread_id))) {
-    stats = g_slice_new0 (GstThreadStats);
+  if (!(stats = g_private_get (&thread_stats_key))) {
+    stats = g_new0 (GstThreadStats, 1);
     stats->tvs_thread = make_trace_values (GST_SECOND);
-    g_hash_table_insert (self->threads, thread_id, stats);
+    g_private_set (&thread_stats_key, stats);
   }
   stats->tthread = tthread;
 
@@ -256,12 +261,38 @@ do_stats (GstTracer * obj, guint64 ts)
 
 /* tracer class */
 
+static void
+gst_rusage_tracer_constructed (GObject * object)
+{
+  GstRUsageTracer *self = GST_RUSAGE_TRACER (object);
+  gchar *params, *tmp;
+  const gchar *name;
+  GstStructure *params_struct = NULL;
+
+  g_object_get (self, "params", &params, NULL);
+
+  if (!params)
+    return;
+
+  tmp = g_strdup_printf ("rusage,%s", params);
+  g_free (params);
+  params_struct = gst_structure_from_string (tmp, NULL);
+  g_free (tmp);
+  if (!params_struct)
+    return;
+
+  /* Set the name if assigned */
+  name = gst_structure_get_string (params_struct, "name");
+  if (name)
+    gst_object_set_name (GST_OBJECT (self), name);
+  gst_structure_free (params_struct);
+}
+
 static void
 gst_rusage_tracer_finalize (GObject * obj)
 {
   GstRUsageTracer *self = GST_RUSAGE_TRACER (obj);
 
-  g_hash_table_destroy (self->threads);
   free_trace_values (self->tvs_proc);
 
   G_OBJECT_CLASS (parent_class)->finalize (obj);
@@ -272,6 +303,7 @@ gst_rusage_tracer_class_init (GstRUsageTracerClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->constructed = gst_rusage_tracer_constructed;
   gobject_class->finalize = gst_rusage_tracer_finalize;
 
   if ((num_cpus = sysconf (_SC_NPROCESSORS_ONLN)) == -1) {
@@ -346,6 +378,9 @@ gst_rusage_tracer_class_init (GstRUsageTracerClass * klass)
           NULL),
       NULL);
   /* *INDENT-ON* */
+
+  GST_OBJECT_FLAG_SET (tr_thread, GST_OBJECT_FLAG_MAY_BE_LEAKED);
+  GST_OBJECT_FLAG_SET (tr_proc, GST_OBJECT_FLAG_MAY_BE_LEAKED);
 }
 
 static void
@@ -368,7 +403,6 @@ gst_rusage_tracer_init (GstRUsageTracer * self)
     gst_tracing_register_hook (tracer, hooks[i], G_CALLBACK (do_stats));
   }
 
-  self->threads = g_hash_table_new_full (NULL, NULL, NULL, free_thread_stats);
   self->tvs_proc = make_trace_values (GST_SECOND);
   self->main_thread_id = g_thread_self ();
 
